from unittest import IsolatedAsyncioTestCase, mock
from .. import captions
from datetime import timedelta
import freezegun


class TestCaptions(IsolatedAsyncioTestCase):
    def assert_is_arthur(self, captionfile):
        self.assertIsNotNone(captionfile)
        self.assertEqual(
            captionfile.events[0].text,
            "Arthur the Rat, read by Brian Goggin from Ireland.",
        )

    def test_autoload(self):
        captionfile = captions.load(
            None, "zoom_narrator/test/fixtures/arthur the rat.mp3"
        )

        self.assert_is_arthur(captionfile)

    def test_manual_load(self):
        captionfile = captions.load(
            "zoom_narrator/test/fixtures/arthur the rat.vtt", "some/nonsense"
        )

        self.assert_is_arthur(captionfile)

    def test_dump(self):
        captionfile = captions.load(
            None, "zoom_narrator/test/fixtures/arthur the rat.mp3"
        )

        self.assertIn(
            "0.005: Arthur the Rat, read by Brian Goggin from Ireland.",
            captions.dump_captions(captionfile),
        )

    async def test_timed_captions(self):
        with freezegun.freeze_time() as frozen_time:
            with mock.patch(
                "asyncio.sleep",
                side_effect=lambda x: frozen_time.tick(timedelta(seconds=x)),
            ) as aisleep:
                captionfile = captions.load(
                    "zoom_narrator/test/fixtures/very_simple_subs.srt", "some/nonsense"
                )

                caption_list = [
                    caption.plaintext
                    async for caption in captions.timed_captions(captionfile)
                ]

                self.assertEqual(
                    ["First Subtitle", "Second Subtitle", "Third Subtitle"],
                    caption_list,
                )

                aisleep.assert_has_awaits(
                    [mock.call(0), mock.call(10.001), mock.call(10)]
                )



from typing import Optional

import asyncclick as click

from . import audio, captions, zoom


@click.command()
@click.argument(
    "audio_path",
    required=True,
    type=click.Path(exists=True),
)
@click.option(
    "-c",
    "--captions",
    "caption_path",
    required=False,
    default=None,
    help="Manually specify captions file.",
    type=click.Path(exists=True),
)
@click.option(
    "-k",
    "--key",
    required=True,
    prompt=True,
    help="Zoom captioning API key (URL). Will prompt if not specified.",
)
async def main(audio_path: str, key: str, caption_path: Optional[str]):
    try:
        caption_file = captions.load(caption_path, audio_path)
    except ValueError as e:
        raise click.ClickException(e)

    with audio.play(audio_path):
        async with zoom.open_zoom_session(key) as session:
            async for event in captions.timed_captions(caption_file):
                await session.send_caption(event.plaintext)


if __name__ == "__main__":
    # execute only if run as a script
    main()



import asyncio
import os
from datetime import datetime, timedelta
from functools import partial
from typing import Optional, AsyncGenerator

import pysubs2

CAPTIONS_EXTENSIONS = ["ssa", "ass", "tmp", "vtt", "srt"]


def _probe(audio_path: str) -> Optional[str]:
    audio_name = _root_name(audio_path)
    audio_dir = os.path.dirname(audio_path)

    for file in os.listdir(audio_dir):
        if (
            _root_name(file) == audio_name
            and _last_extension(file) in CAPTIONS_EXTENSIONS
        ):
            return os.path.join(audio_dir, file)

    return None


def load(caption_path: Optional[str], audio_path: str) -> pysubs2.SSAFile:
    if not caption_path:
        caption_path = _probe(audio_path)
    if not caption_path:
        raise ValueError("Couldn't find caption file, please specify manually.")

    return pysubs2.load(caption_path)


def dump_captions(captions: pysubs2.SSAFile) -> str:
    return "\n".join(
        [f"{event.start/1000:05}: {event.text}" for event in captions.events]
    )


async def timed_captions(
    captions: pysubs2.SSAFile,
) -> AsyncGenerator[pysubs2.SSAEvent, None]:
    wait_until_event_time = partial(_wait_until_reltime, datetime.now())

    for event in captions:
        await wait_until_event_time(event.start)
        yield event


async def _wait_until_reltime(base_time: datetime, reltime_ms: int) -> None:
    abs_time = base_time + timedelta(milliseconds=reltime_ms)
    time_until = abs_time - datetime.now()

    # If the system ever gets delayed and time_until winds up being less than zero, events will just start triggering
    # as soon as they're queued until things get caught up.
    await asyncio.sleep(time_until.total_seconds())


def _root_name(path):
    return os.path.basename(path).split(".")[0]


def _last_extension(path):
    return os.path.basename(path).split(".")[-1]



import json
import os
from contextlib import asynccontextmanager

from aiohttp import ClientSession

RESUME_FILE = ".resumesession"


@asynccontextmanager
async def open_zoom_session(api_key):
    async with ClientSession() as http_session:
        session = ZoomSession(http_session, api_key)
        try:
            yield session
        finally:
            session.close()


class ZoomSession:
    def __init__(self, http_session, api_key, seq_start=0):
        self.http_session = http_session
        self.api_key = api_key

        # The closed captioning API is very minimalistic, and one limitation that creates is the inability to reset the sequence
        # number. Hence if more than one connection is going to be made to a given meeting over its lifetime, we need to
        # remember where we left off or it will get unhappy with our "out-of-order" data and delay new lines until some timeout
        # expires.
        self._seq = self._load_resume_seq()

    @property
    def seq(self):
        current = self._seq
        self._seq += 1
        return current

    async def send_caption(self, caption: str):
        await self.http_session.post(
            self.api_key,
            params={"seq": self.seq, "lang": "en-US"},
            data=caption,
        )

        print(caption)

    def _load_resume_seq(self):
        resume_seq = 0
        if os.path.exists(RESUME_FILE):
            with open(RESUME_FILE) as resume_file:
                saved = json.load(resume_file)
                assert "seq", "api_key" in saved.keys()
                if saved["api_key"] == self.api_key:
                    resume_seq = saved["seq"]

        return resume_seq

    def _save_resume_seq(self):
        with open(RESUME_FILE, "w") as resume_file:
            json.dump({"api_key": self.api_key, "seq": self._seq}, resume_file)

    def close(self):
        self._save_resume_seq()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['zoom_narrator', 'zoom_narrator.test']

package_data = \
{'': ['*'], 'zoom_narrator.test': ['fixtures/*']}

install_requires = \
['aiohttp[speedups]>=3.7.3,<4.0.0',
 'asyncclick>=7.1.2,<8.0.0',
 'pysubs2>=1.0.0,<2.0.0']

entry_points = \
{'console_scripts': ['zoom-narrator = zoom_narrator.main:main']}

setup_kwargs = {
    'name': 'zoom-narrator',
    'version': '1.0.1',
    'description': 'Play audio while streaming its closed captions to Zoom',
    'long_description': None,
    'author': 'Daniel Maxwell-Ross',
    'author_email': 'daniel@maxwell-ross.us',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
